#import "EPCData.h"
#import "GS1EPCRFIDSGTINData.h"
@implementation EPCData {
}

- (instancetype)initWithEpcData:(NSData *)epcData{
    if(!epcData||[epcData length] == 0){
         return nil;
    }
	unsigned char* ptr= (unsigned char*) [epcData bytes];
	NSString *hexStr = [NSString stringWithFormat:@"%02X", *ptr++ & 0xFF ];
	EPCHeader *epcHeader =[[EPCHeader alloc]initWithHexHeaderValue:[hexStr copy]];
	if (!epcHeader.hexHeaderValue || [epcHeader.hexHeaderValue isEqualToString:@""]) {
		return nil;
	}else{
       if ([@"SGTIN-96" isEqualToString:[epcHeader codingScheme]]) {
            if ((epcData.length*8)<96) {
                return nil;
            }
			GS1EPCRFIDSGTINData *gs1Sgtin = [[GS1EPCRFIDSGTINData alloc]init];
			[gs1Sgtin setSgtinType:SGTINTypeSGTIN_96];
			self = gs1Sgtin;
		}else{
			return nil;
		}
	}
	if (self) {
		_header = epcHeader;
		_epcData = epcData;
		[self setEpcData:epcData];
	}
	return self;
}

-(void)writeEPCData:(NSData*)epcData{
    _epcData=epcData;
}
- (BOOL)setHexEpcValue:(NSString *)hexEpcValue dataLength:(NSUInteger)dataLength{
    return NO;
}

- (void)setEpcData:(NSData *)epcData{
    @try {
        if (epcData && epcData.length > 0) {
            self.dataBits = epcData.length * 8;
            NSMutableString* hexStr = [[NSMutableString alloc] init];
            unsigned char* ptr= (unsigned char*) [epcData bytes];
            for(NSUInteger i = 1; i <= epcData.length; i++) {
                [hexStr appendFormat:@"%02X", *ptr++ & 0xFF ];
				if (i != 1) {
					if (![self setHexEpcValue:hexStr dataLength:i]) {
						return;
					}
				}
            }
            _epcData = epcData;
            self.hexEpcValue = hexStr;
        }else{
            @throw [[NSException alloc] initWithName:@"EpcData data error" reason:@"Lack the necessary information" userInfo:nil];
        }
    } @catch (NSException *exception) {
        [self initData];
    } @finally {
        
    }
}
+(NSString *)getBinaryByhex:(NSString *)hex
{
    NSMutableDictionary  *hexDic = [[NSMutableDictionary alloc] init];
	
    hexDic = [[NSMutableDictionary alloc] initWithCapacity:16];
	
    [hexDic setObject:@"0000" forKey:@"0"];
	
    [hexDic setObject:@"0001" forKey:@"1"];
	
    [hexDic setObject:@"0010" forKey:@"2"];
    
    [hexDic setObject:@"0011" forKey:@"3"];
    
    [hexDic setObject:@"0100" forKey:@"4"];
    
    [hexDic setObject:@"0101" forKey:@"5"];
    
    [hexDic setObject:@"0110" forKey:@"6"];
    
    [hexDic setObject:@"0111" forKey:@"7"];
    
    [hexDic setObject:@"1000" forKey:@"8"];
    
    [hexDic setObject:@"1001" forKey:@"9"];
    
    [hexDic setObject:@"1010" forKey:@"A"];
    
    [hexDic setObject:@"1011" forKey:@"B"];
    
    [hexDic setObject:@"1100" forKey:@"C"];
    
    [hexDic setObject:@"1101" forKey:@"D"];
    
    [hexDic setObject:@"1110" forKey:@"E"];
    
    [hexDic setObject:@"1111" forKey:@"F"];
    
    NSString *binaryString=[[NSString alloc] init];
    
    for (int i=0; i<[hex length]; i++) {
        
        NSRange rage;
        
        rage.length = 1;
        
        rage.location = i;
        
        NSString *key = [hex substringWithRange:rage];
        
        
        binaryString = [NSString stringWithFormat:@"%@%@",binaryString,[NSString stringWithFormat:@"%@",[hexDic objectForKey:key]]];
        
    }
    
    return binaryString;
    
}
- (NSString *)upcCD{
    if (self.rfidProductCD && self.rfidProductCD.length == 14) {
        NSString *string = [NSString stringWithFormat:@"%@ %@ %@ %@",[self.rfidProductCD substringWithRange:NSMakeRange(0, 1)],[self.rfidProductCD substringWithRange:NSMakeRange(1, 5)],[self.rfidProductCD substringWithRange:NSMakeRange(6, 5)],[self.rfidProductCD substringWithRange:NSMakeRange(11, 1)]];
        return string;

    }
    return nil;
}
- (void)initData{
 	_epcData = nil;
	_hexEpcValue = @"";
	_dataBits = 0;
 	_header = nil;
 	_rfidProductCD = @"";
	_epcPureURI = @"";
	_epcTagURI = @"";
	_barcodeType = @"";
} 
@end

@implementation EPCHeader
- (instancetype)initWithHexHeaderValue:(NSString *)hexHeaderValue{
	self = [super init];
	if (self) {
		self.hexHeaderValue = hexHeaderValue;
	}
	return self;
}
- (void)setHexHeaderValue:(NSString *)hexHeaderValue{
     long long a =  strtoull([hexHeaderValue UTF8String], 0, 16);
	if ((!hexHeaderValue || [hexHeaderValue isEqualToString:@""])|| a > 0xff) {
		[self initData];
		return;
	}
	@try {
        NSDictionary *headerDic = @{@"01":@[@"NA",@"Reserved for Future Use"],
                                  @"08":@[@"64",@"Reserved until 64bit Sunset <SSCC-64>"],
                                  @"0C":@[@"",@"Reserved until 64 bit Sunset Due to 64 bit encoding rule in Gen 1"],
                                  @"10":@[@"NA",@"Reserved for Future Use"],
                                  @"2C":@[@"NA",@"GDTI-96"],
                                  @"2D":@[@"96",@"GSRN-96"],
                                  @"2E":@[@"NA",@"Reserved for Future Use"],
                                @"2F":@[@"96",@"USDoD-96"],
                                @"30":@[@"96",@"SGTIN-96"],
                                  @"31":@[@"96",@"SSCC-96"],
                                  @"32":@[@"96",@"SGLN-96"],
                                  @"33":@[@"96",@"GRAI-96"],
                                  @"34":@[@"96",@"GIAI-96"],
                                  @"35":@[@"96",@"GID-96"],
                              @"36":@[@"198",@"SGTIN-198"],
                                @"37":@[@"170",@"GRAI-170"],
                                @"38":@[@"202",@"GIAI-202"],
                                @"39":@[@"195",@"SGLN-195"],
                                @"3A":@[@"113",@"GDTI-113"],
                                @"3B":@[@"Variable",@"ADI-var"],
                            @"3C":@[@"NA",@"Reserved for future Header values"],
                                @"40":@[@"",@"Reserved until 64 bit Sunset"],
                                @"":@[@"",@""],
                                @"C0":@[@"",@"Reserved until 64 bit Sunset <SGTIN-64> (64 header values)"],
                                @"CE":@[@"64",@"Reserved until 64 bit Sunset <DoD-64>"],
                                @"CF":@[@"",@"Reserved until 64 bit Sunset Following 64 bit Sunset, E2 remains reserved to avoid confusion with the first eight bits of TID memory (Section 16)."],
                                  @"FF":@[@"NA",@"Reserved for future headers longer than 8 bits"],
                            @"00":@[@"UNIQLO",@"UNIQLO"],
                                  };
		NSArray *headerValue = [headerDic objectForKey:hexHeaderValue];
		if (headerValue && [headerValue count] > 0 ) {
			_hexHeaderValue = hexHeaderValue;
			_encodingLength = [[headerValue objectAtIndex:0] longLongValue];
			_codingScheme = [headerValue lastObject];
		}else{
            _hexHeaderValue = hexHeaderValue;
            _encodingLength = [@"NA" longLongValue];
            _codingScheme = @"Unprogrammed Tag";
		}
		
	}
	@catch (NSException *exception) {
		[self initData];
		return;
	}
	@finally {
		
	}
}

- (void)initData{
	_hexHeaderValue = @"";
	_encodingLength = 0;
	_codingScheme = @"";
	_binaryHeaderValue = @"";
}
@end
